@@ -26,6 +26,9 @@
 extern "C" {
 #endif  /* __cplusplus */
 
+#define GRN_TOKENIZER_TOKENIZED_DELIMITER_UTF8     "\xEF\xBF\xBE"
+#define GRN_TOKENIZER_TOKENIZED_DELIMITER_UTF8_LEN 3
+
 /*
   grn_tokenizer_charlen() returns the length (#bytes) of the first character
   in the string specified by `str_ptr' and `str_length'. If the starting bytes
@@ -43,6 +46,26 @@ int grn_tokenizer_charlen(grn_ctx *ctx, const char *str_ptr,
 int grn_tokenizer_isspace(grn_ctx *ctx, const char *str_ptr,
                           unsigned int str_length, grn_encoding encoding);
 
+/*
+  grn_tokenizer_is_tokenized_delimiter() returns whether is the first
+  character in the string specified by `str_ptr' and `str_length' the
+  special tokenized delimiter character or not.
+ */
+grn_bool grn_tokenizer_is_tokenized_delimiter(grn_ctx *ctx,
+                                              const char *str_ptr,
+                                              unsigned int str_length,
+                                              grn_encoding encoding);
+
+/*
+  grn_tokenizer_have_tokenized_delimiter() returns whether is there
+  the special delimiter character in the string specified by `str_ptr'
+  and `str_length' the special tokenized delimiter character or not.
+ */
+GRN_PLUGIN_EXPORT grn_bool grn_tokenizer_have_tokenized_delimiter(grn_ctx *ctx,
+                                                                  const char *str_ptr,
+                                                                  unsigned int str_length,
+                                                                  grn_encoding encoding);
+
 /*
   grn_tokenizer_query is a structure for storing a query. See the following
   functions.
@@ -58,20 +81,34 @@ struct _grn_tokenizer_query {
 };
 
 /*
-  grn_tokenizer_query_create() parses `args' and returns a new object of
+  grn_tokenizer_query_open() parses `args' and returns a new object of
   grn_tokenizer_query. The new object stores information of the query.
   grn_tokenizer_query_create() normalizes the query if the target table
   requires normalization. grn_tokenizer_query_create() returns NULL if
   something goes wrong. Note that grn_tokenizer_query_create() must be called
   just once in the function that initializes a tokenizer.
  */
+GRN_PLUGIN_EXPORT grn_tokenizer_query *grn_tokenizer_query_open(grn_ctx *ctx,
+                                                                int num_args, grn_obj **args);
+
+/*
+  grn_tokenizer_query_create() is deprecated. Use grn_tokenizer_query_open()
+  instead.
+*/
+
 grn_tokenizer_query *grn_tokenizer_query_create(grn_ctx *ctx,
                                                 int num_args, grn_obj **args);
 
 /*
-  grn_tokenizer_mutex_destroy() finalizes an object of grn_tokenizer_mutex
+  grn_tokenizer_query_close() finalizes an object of grn_tokenizer_query
   and then frees memory allocated for that object.
  */
+GRN_PLUGIN_EXPORT void grn_tokenizer_query_close(grn_ctx *ctx, grn_tokenizer_query *query);
+
+/*
+  grn_tokenizer_query_destroy() is deprecated. Use grn_tokenizer_query_close()
+  instead.
+ */
 void grn_tokenizer_query_destroy(grn_ctx *ctx, grn_tokenizer_query *query);
 
 /*
@@ -90,13 +127,13 @@ struct _grn_tokenizer_token {
   grn_tokenizer_token_init() initializes `token'. Note that an initialized
   object must be finalized by grn_tokenizer_token_fin().
  */
-void grn_tokenizer_token_init(grn_ctx *ctx, grn_tokenizer_token *token);
+GRN_PLUGIN_EXPORT void grn_tokenizer_token_init(grn_ctx *ctx, grn_tokenizer_token *token);
 
 /*
   grn_tokenizer_token_fin() finalizes `token' that has been initialized by
   grn_tokenizer_token_init().
  */
-void grn_tokenizer_token_fin(grn_ctx *ctx, grn_tokenizer_token *token);
+GRN_PLUGIN_EXPORT void grn_tokenizer_token_fin(grn_ctx *ctx, grn_tokenizer_token *token);
 
 /*
   grn_tokenizer_status provides a list of tokenizer status codes.
@@ -111,16 +148,29 @@ typedef enum {
 } grn_tokenizer_status;
 
 /*
-  grn_tokenizer_token_push() pushes the next token in `*token'. Note that
+  grn_tokenizer_token_push() pushes the next token into `token'. Note that
   grn_tokenizer_token_push() does not make a copy of the given string. This
   means that you have to maintain a memory space allocated to the string.
   Also note that the grn_tokenizer_token object must be maintained until the
   request for the next token or finalization comes. See grn_tokenizer_status in
   this header for more details of `status'.
  */
-void grn_tokenizer_token_push(grn_ctx *ctx, grn_tokenizer_token *token,
-                              const char *str_ptr, unsigned int str_length,
-                              grn_tokenizer_status status);
+GRN_PLUGIN_EXPORT void grn_tokenizer_token_push(grn_ctx *ctx, grn_tokenizer_token *token,
+                                                const char *str_ptr, unsigned int str_length,
+                                                grn_tokenizer_status status);
+
+/*
+  grn_tokenizer_tokenized_delimiter_next() extracts the next token
+  from the string specified by `str_ptr' and `str_length' and pushes
+  the next token into `token'. It returns the string after the next
+  token. The returned string may be `NULL' when all tokens are
+  extracted.
+ */
+GRN_PLUGIN_EXPORT const char *grn_tokenizer_tokenized_delimiter_next(grn_ctx *ctx,
+                                                                     grn_tokenizer_token *token,
+                                                                     const char *str_ptr,
+                                                                     unsigned int str_length,
+                                                                     grn_encoding encoding);
 
 /*
   grn_tokenizer_register() registers a plugin to the database which is
@@ -134,10 +184,10 @@ void grn_tokenizer_token_push(grn_ctx *ctx, grn_tokenizer_token *token,
   details of grn_proc_func and grn_user_data, that is used as an argument of
   grn_proc_func.
  */
-grn_rc grn_tokenizer_register(grn_ctx *ctx, const char *plugin_name_ptr,
-                              unsigned int plugin_name_length,
-                              grn_proc_func *init, grn_proc_func *next,
-                              grn_proc_func *fin);
+GRN_PLUGIN_EXPORT grn_rc grn_tokenizer_register(grn_ctx *ctx, const char *plugin_name_ptr,
+                                                unsigned int plugin_name_length,
+                                                grn_proc_func *init, grn_proc_func *next,
+                                                grn_proc_func *fin);
 
 #ifdef __cplusplus
 }  /* extern "C" */
