@@ -24,6 +24,7 @@
 #include "dat.h"
 #include "hash.h"
 #include "string_in.h"
+#include <groonga/tokenizer.h>
 
 grn_obj *grn_uvector_tokenizer = NULL;
 
@@ -83,14 +84,11 @@ typedef struct {
   grn_obj *nstr;
   const uint8_t *delimiter;
   uint32_t delimiter_len;
-  int32_t pos;
   grn_encoding encoding;
   const unsigned char *next;
   const unsigned char *end;
-  int32_t len;
-  uint32_t tail;
-  grn_obj curr_;
-  grn_obj stat_;
+  grn_tokenizer_token token;
+  grn_bool have_tokenized_delimiter;
 } grn_delimited_tokenizer;
 
 static grn_obj *
@@ -102,69 +100,97 @@ delimited_init(grn_ctx *ctx, grn_obj *table, grn_user_data *user_data,
   int nflags = 0;
   const char *normalized;
   unsigned int normalized_length_in_bytes;
-  grn_delimited_tokenizer *token;
+  grn_delimited_tokenizer *tokenizer;
   grn_obj_flags table_flags;
   if (!(str = grn_ctx_pop(ctx))) {
     ERR(GRN_INVALID_ARGUMENT, "missing argument");
     return NULL;
   }
-  if (!(token = GRN_MALLOC(sizeof(grn_delimited_tokenizer)))) { return NULL; }
-  user_data->ptr = token;
-  token->delimiter = delimiter;
-  token->delimiter_len = delimiter_len;
-  token->pos = 0;
-  grn_table_get_info(ctx, table, &table_flags, &token->encoding, NULL);
+  if (!(tokenizer = GRN_MALLOC(sizeof(grn_delimited_tokenizer)))) {
+    return NULL;
+  }
+  user_data->ptr = tokenizer;
+
+  grn_table_get_info(ctx, table, &table_flags, &tokenizer->encoding, NULL);
+
+  tokenizer->have_tokenized_delimiter =
+    grn_tokenizer_have_tokenized_delimiter(ctx,
+                                           GRN_TEXT_VALUE(str),
+                                           GRN_TEXT_LEN(str),
+                                           tokenizer->encoding);
+  tokenizer->delimiter = delimiter;
+  tokenizer->delimiter_len = delimiter_len;
+
   if (table_flags & GRN_OBJ_KEY_NORMALIZE) {
     normalizer = GRN_NORMALIZER_AUTO;
   }
-  if (!(token->nstr = grn_string_open_(ctx,
-                                       GRN_TEXT_VALUE(str), GRN_TEXT_LEN(str),
-                                       normalizer, nflags, token->encoding))) {
-    GRN_FREE(token);
+  tokenizer->nstr = grn_string_open_(ctx,
+                                     GRN_TEXT_VALUE(str), GRN_TEXT_LEN(str),
+                                     normalizer, nflags, tokenizer->encoding);
+  if (!tokenizer->nstr) {
+    GRN_FREE(tokenizer);
     ERR(GRN_TOKENIZER_ERROR, "grn_string_open failed at grn_token_open");
     return NULL;
   }
-  grn_string_get_normalized(ctx, token->nstr,
+  grn_string_get_normalized(ctx, tokenizer->nstr,
                             &normalized, &normalized_length_in_bytes,
-                            &(token->len));
-  token->next = (const unsigned char *)normalized;
-  token->end = token->next + normalized_length_in_bytes;
-  GRN_TEXT_INIT(&token->curr_, GRN_OBJ_DO_SHALLOW_COPY);
-  GRN_UINT32_INIT(&token->stat_, 0);
+                            NULL);
+  tokenizer->next = (const unsigned char *)normalized;
+  tokenizer->end = tokenizer->next + normalized_length_in_bytes;
+
+  grn_tokenizer_token_init(ctx, &(tokenizer->token));
+
   return NULL;
 }
 
 static grn_obj *
 delimited_next(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  size_t cl;
-  grn_delimited_tokenizer *token = user_data->ptr;
-  const unsigned char *p = token->next, *r;
-  const unsigned char *e = token->end;
-  for (r = p; r < e; r += cl) {
-    if (!(cl = grn_charlen_(ctx, (char *)r, (char *)e, token->encoding))) {
-      token->next = (unsigned char *)e;
-      break;
+  grn_delimited_tokenizer *tokenizer = user_data->ptr;
+
+  if (tokenizer->have_tokenized_delimiter) {
+    unsigned int rest_length;
+    rest_length = tokenizer->end - tokenizer->next;
+    tokenizer->next =
+      grn_tokenizer_tokenized_delimiter_next(ctx,
+                                             &(tokenizer->token),
+                                             tokenizer->next,
+                                             rest_length,
+                                             tokenizer->encoding);
+  } else {
+    size_t cl;
+    const unsigned char *p = tokenizer->next, *r;
+    const unsigned char *e = tokenizer->end;
+    grn_tokenizer_status status;
+    for (r = p; r < e; r += cl) {
+      if (!(cl = grn_charlen_(ctx, (char *)r, (char *)e, tokenizer->encoding))) {
+        tokenizer->next = (unsigned char *)e;
+        break;
+      }
+      if (r + tokenizer->delimiter_len <= e &&
+          !memcmp(r, tokenizer->delimiter, tokenizer->delimiter_len)) {
+        tokenizer->next = r + tokenizer->delimiter_len;
+        break;
+      }
     }
-    if (r + token->delimiter_len <= e &&
-        !memcmp(r, token->delimiter, token->delimiter_len)) {
-      token->next = r + token->delimiter_len;
-      break;
+    if (r == e) {
+      status = GRN_TOKENIZER_LAST;
+    } else {
+      status = GRN_TOKENIZER_CONTINUE;
     }
+    grn_tokenizer_token_push(ctx, &(tokenizer->token), p, r - p, status);
   }
-  GRN_TEXT_SET_REF(&token->curr_, p, r - p);
-  GRN_UINT32_SET(ctx, &token->stat_, r == e ? GRN_TOKEN_LAST : 0);
-  grn_ctx_push(ctx, &token->curr_);
-  grn_ctx_push(ctx, &token->stat_);
+
   return NULL;
 }
 
 static grn_obj *
 delimited_fin(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  grn_delimited_tokenizer *token = user_data->ptr;
-  grn_obj_close(ctx, token->nstr);
-  GRN_FREE(token);
+  grn_delimited_tokenizer *tokenizer = user_data->ptr;
+  grn_obj_close(ctx, tokenizer->nstr);
+  grn_tokenizer_token_fin(ctx, &(tokenizer->token));
+  GRN_FREE(tokenizer);
   return NULL;
 }
 
@@ -212,7 +238,10 @@ ngram_init(grn_ctx *ctx, grn_obj *table, grn_user_data *user_data, uint8_t ngram
 {
   grn_obj *str;
   grn_obj *normalizer = NULL;
-  int nflags = GRN_STRING_REMOVE_BLANK|GRN_STRING_WITH_TYPES;
+  int nflags =
+    GRN_STRING_REMOVE_BLANK |
+    GRN_STRING_WITH_TYPES |
+    GRN_STRING_REMOVE_TOKENIZED_DELIMITER;
   const char *normalized;
   unsigned int normalized_length_in_bytes;
   grn_ngram_tokenizer *token;
@@ -471,7 +500,7 @@ grn_token_open(grn_ctx *ctx, grn_obj *table, const char *str, size_t str_len,
     }
   }
   if (ctx->rc) {
-    GRN_FREE(token);
+    grn_token_close(ctx, token);
     token = NULL;
   }
   return token;
