@@ -1428,6 +1428,9 @@ g_output(grn_ctx *ctx, int flags, void *arg)
   msg->edge_id = req->edge_id;
   msg->header.proto = req->header.proto == GRN_COM_PROTO_MBREQ
     ? GRN_COM_PROTO_MBRES : req->header.proto;
+  if (ctx->rc != GRN_SUCCESS && GRN_BULK_VSIZE(ctx->impl->outbuf) == 0) {
+    grn_ctx_output_cstr(ctx, ctx->errbuf);
+  }
   if (grn_msg_send(ctx, (grn_obj *)msg,
                    (flags & GRN_CTX_MORE) ? GRN_CTX_MORE : GRN_CTX_TAIL)) {
     edge->stat = EDGE_ABORT;
@@ -1775,6 +1778,29 @@ static grn_command_version default_default_command_version =
 static int64_t default_default_match_escalation_threshold = 0;
 static const char * const default_bind_address = "0.0.0.0";
 
+static void
+init_default_hostname(void)
+{
+  static char hostname[HOST_NAME_MAX + 1];
+  struct addrinfo hints, *result;
+
+  hostname[HOST_NAME_MAX] = '\0';
+  if (gethostname(hostname, HOST_NAME_MAX) == -1)
+    return;
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_addr = NULL;
+  hints.ai_canonname = NULL;
+  hints.ai_next = NULL;
+  if (getaddrinfo(hostname, NULL, &hints, &result) != 0)
+    return;
+  freeaddrinfo(result);
+
+  default_hostname = hostname;
+}
+
 static void
 init_default_settings(void)
 {
@@ -1789,29 +1815,7 @@ init_default_settings(void)
     }
   }
 
-  {
-    static char hostname[HOST_NAME_MAX + 1];
-    hostname[HOST_NAME_MAX] = '\0';
-    if (gethostname(hostname, HOST_NAME_MAX)) {
-      fprintf(stderr, "gethostname failed: %s\n", strerror(errno));
-    } else {
-      int error_code;
-      struct addrinfo hints, *result;
-      memset(&hints, 0, sizeof(hints));
-      hints.ai_family = AF_UNSPEC;
-      hints.ai_socktype = SOCK_STREAM;
-      hints.ai_addr = NULL;
-      hints.ai_canonname = NULL;
-      hints.ai_next = NULL;
-      error_code = getaddrinfo(hostname, NULL, &hints, &result);
-      if (error_code) {
-        fprintf(stderr, "getaddrinfo failed: %s\n", gai_strerror(error_code));
-      } else {
-        freeaddrinfo(result);
-        default_hostname = hostname;
-      }
-    }
-  }
+  init_default_hostname();
 
   if (grn_log_path) {
     default_log_path = grn_log_path;
