@@ -31,6 +31,8 @@ void data_normalize(void);
 void test_normalize(gconstpointer data);
 void data_normalize_broken(void);
 void test_normalize_broken(gconstpointer data);
+void data_remove_tokenized_delimiter(void);
+void test_remove_tokenized_delimiter(gconstpointer data);
 void data_charlen_broken(void);
 void test_charlen_broken(gconstpointer data);
 void data_urlenc(void);
@@ -260,6 +262,69 @@ test_normalize_broken(gconstpointer data)
   grn_test_assert(grn_str_close(&context, string));
 }
 
+void
+data_remove_tokenized_delimiter(void)
+{
+#define ADD_DATUM(label, expected, input, flags)                        \
+  gcut_add_datum(label,                                                 \
+                 "expected", G_TYPE_STRING, expected,                   \
+                 "input", G_TYPE_STRING, input,                         \
+                 "flags", G_TYPE_INT, flags,                            \
+                 NULL)
+
+#define UFFFE_IN_UTF8 "\xef\xbf\xbe"
+
+  ADD_DATUM("normalize",
+            "abあい",
+            UFFFE_IN_UTF8 "A"
+            UFFFE_IN_UTF8 "B"
+            UFFFE_IN_UTF8 "あ"
+            UFFFE_IN_UTF8 "い"
+            UFFFE_IN_UTF8,
+            GRN_OBJ_KEY_NORMALIZE);
+  ADD_DATUM("not normalize",
+            "ABあい",
+            UFFFE_IN_UTF8 "A"
+            UFFFE_IN_UTF8 "B"
+            UFFFE_IN_UTF8 "あ"
+            UFFFE_IN_UTF8 "い"
+            UFFFE_IN_UTF8,
+            0);
+
+#undef UFFFE_IN_UTF8
+
+#undef ADD_DATUM
+}
+
+void
+test_remove_tokenized_delimiter(gconstpointer data)
+{
+  grn_obj *string;
+  grn_obj *normalizer = NULL;
+  const gchar *expected;
+  const gchar *input;
+  const gchar *normalized;
+  unsigned int length_in_bytes;
+  int flags = GRN_STRING_REMOVE_TOKENIZED_DELIMITER;
+
+  GRN_CTX_SET_ENCODING(&context, GRN_ENC_UTF8);
+
+  input = gcut_data_get_string(data, "input");
+  flags |= gcut_data_get_int(data, "flags");
+  if (flags & GRN_OBJ_KEY_NORMALIZE) {
+    normalizer = GRN_NORMALIZER_AUTO;
+  }
+
+  string = grn_string_open(&context, input, strlen(input), normalizer, flags);
+  grn_string_get_normalized(&context, string,
+                            &normalized, &length_in_bytes, NULL);
+  normalized = cut_take_strndup(normalized, length_in_bytes);
+  grn_obj_unlink(&context, string);
+
+  expected = gcut_data_get_string(data, "expected");
+  cut_assert_equal_string(expected, normalized);
+}
+
 void
 data_charlen_broken(void)
 {
@@ -746,11 +811,11 @@ data_str_len(void)
 #define ADD_DATUM_ALL_ENCODING(label, expected, input)  \
   ADD_DATUM(label " (none) <" input ">",                \
             expected, input, GRN_ENC_NONE);             \
-  ADD_DATUM(label " (eucjP) <" input ">",               \
+  ADD_DATUM(label " (EUC-JP) <" input ">",              \
             expected, input, GRN_ENC_EUC_JP);           \
   ADD_DATUM(label " (UTF-8) <" input ">",               \
             expected, input, GRN_ENC_UTF8);             \
-  ADD_DATUM(label " (Shift_JIS-8) <" input ">",         \
+  ADD_DATUM(label " (Shift_JIS) <" input ">",           \
             expected, input, GRN_ENC_SJIS);             \
   ADD_DATUM(label " (Latin1) <" input ">",              \
             expected, input, GRN_ENC_LATIN1);           \
@@ -758,7 +823,7 @@ data_str_len(void)
             expected, input, GRN_ENC_KOI8R);
 
 #define ADD_DATUM_JAPANESE(label, expected, input)                      \
-  ADD_DATUM("Japanese: " label " (eucjP) <" input ">",                  \
+  ADD_DATUM("Japanese: " label " (EUC-JP) <" input ">",                 \
             expected, cut_take_convert(input, "eucJP", "UTF-8"),        \
             GRN_ENC_EUC_JP);                                            \
   ADD_DATUM("Japanese: " label " (UTF-8) <" input ">",                  \
